#include "femMedium.h"
#include "femProblem.h"
#include "femGroup.h"

namespace fem
{

Medium::Medium(Problem &pbl, std::string const &groupname,
               double E, double nu) : m_E(E), m_nu(nu), H2D(3, 3), H3D(6, 6)
{
    if (pbl.entities.empty()) // is the problem already sync'ed?
        pbl.sync();

    group = pbl.groups_by_name.at(groupname);
    pbl.media.push_back(this);

    // construct 2D-plane-stress Hooke matrix
    H2D << 1.0, nu, 0.0,
        nu, 1, 0.0,
        0.0, 0.0, (1.0 - nu) / 2.0;
    H2D *= E / (1 - nu * nu);

    // Eigen::IOFormat fmt(6, 0, ", ", "\n", "\t[", "]");
    // std::cout << "H2D =" << H2D.format(fmt);

    // construct 3D Hooke matrix
    H3D << 1.0 - nu, nu, nu, 0.0, 0.0, 0.0,
        nu, 1.0 - nu, nu, 0.0, 0.0, 0.0,
        nu, nu, 1.0 - nu, 0.0, 0.0, 0.0,
        0.0, 0.0, 0.0, (1.0 - 2 * nu) / 2.0, 0.0, 0.0,
        0.0, 0.0, 0.0, 0.0, (1.0 - 2 * nu) / 2.0, 0.0,
        0.0, 0.0, 0.0, 0.0, 0.0, (1.0 - 2 * nu) / 2.0;
    H3D *= E / (1 + nu) / (1 - 2 * nu);
}

Medium::~Medium()
{
    if(debug_level()>0)
        std::cout << "~Medium()\n";
}

FEM_API std::ostream &
operator<<(std::ostream &out, Medium const &obj)
{
    out << "Medium on \"" << obj.group->name
        << "\": E=" << obj.m_E << ", nu=" << obj.m_nu;
    return out;
}

} // namespace fem
